import React, { useEffect, useState } from "react";
import Api from "../utils/api";

import { makeStyles } from "@material-ui/core/styles";
import Table from "@material-ui/core/Table";
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import TableContainer from "@material-ui/core/TableContainer";
import TableHead from "@material-ui/core/TableHead";
import TableRow from "@material-ui/core/TableRow";
import Paper from "@material-ui/core/Paper";
import Pagination from "@material-ui/lab/Pagination";
import CircularProgress from "@material-ui/core/CircularProgress";
import { useHistory } from "react-router-dom";

const useStyles = makeStyles((theme) => ({
  table: {
    minWidth: 650,
    overflowY: "scroll"
  },
  tableContainer: {
    margin: 24
  },
  tableRow: {
    "&:hover": {
      cursor: "pointer",
      backgroundColor: theme.palette.background.default
    }
  }
}));

const createNumbers = (numbers) => [...Array(31)].map((_value, index) => (numbers ? numbers[index + 1] : index + 1));

function createData({ month, year, ...row }) {
  return { month, year, sunValues: createNumbers(row) };
}

const headerTemplate = {
  year: "Year",
  month: "Měsíc",
  labels: createNumbers()
};

const DataTable = ({ rows }) => {
  const history = useHistory();
  const classes = useStyles();
  return (
    <TableContainer className={classes.tableContainer} component={Paper}>
      <Table className={classes.table} aria-label="simple table">
        <TableHead>
          <TableRow>
            <TableCell>{headerTemplate.year}</TableCell>
            <TableCell align="right">{headerTemplate.month}</TableCell>
            {headerTemplate.labels.map((label) => (
              <TableCell key={label}>{label}</TableCell>
            ))}
          </TableRow>
        </TableHead>
        <TableBody>
          {rows.map((data) => {
            const row = createData(data);
            return (
              <TableRow key={row.name} onClick={() => history.push(`/detail/${data._id}`)} className={classes.tableRow}>
                <TableCell component="th" scope="row">
                  {row.year}
                </TableCell>
                <TableCell align="right">{row.month}</TableCell>
                {row.sunValues.map((number, index) => (
                  <TableCell key={`${number}-${index}`}>{number}</TableCell>
                ))}
              </TableRow>
            );
          })}
        </TableBody>
      </Table>
    </TableContainer>
  );
};

const HomePage = () => {
  const [data, setData] = useState([]);
  const [{ max, skip, count }, setPagination] = useState({
    max: 15,
    skip: 0,
    count: 0
  });
  const [initialized, setInitialized] = useState(false);
  const handlePagination = (_event, page) => {
    setPagination((prevPagination) => ({
      ...prevPagination,
      skip: (page - 1) * prevPagination.max
    }));
  };
  const query = `max=${max}&skip=${skip}`;
  useEffect(() => {
    setInitialized(false);
    Promise.all([
      Api.get(`rest/sunshine?q={}&${query}`).then((data) => setData(data)),
      Api.get(`rest/sunshine?q={}&h={"$aggregate":["COUNT:"]}`).then((data) =>
        setPagination((prevPagination) => ({ ...prevPagination, count: data["COUNT "] }))
      )
    ]).then(() => setInitialized(true));
  }, [skip]);

  if (!initialized) {
    return (
      <div>
        <CircularProgress />
      </div>
    );
  }
  return (
    <div>
      <h1>Home page</h1>
      <Pagination count={Math.ceil(count / max)} page={skip / max + 1} onChange={handlePagination} />
      <DataTable rows={data} />
    </div>
  );
};

export default HomePage;
